package com.mobai.iotDB.controller;

import com.mobai.domain.Vehicle;
import com.mobai.iotDB.config.IotDBSessionConfig;
import com.mobai.iotDB.domain.resp.ResponseData;
import com.mobai.iotDB.service.IotDbServer;
import com.mobai.req.VehicleReq;
import lombok.extern.log4j.Log4j2;
import org.apache.iotdb.rpc.IoTDBConnectionException;
import org.apache.iotdb.rpc.StatementExecutionException;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RestController;

import javax.annotation.Resource;
import java.rmi.ServerException;

/**
 * description: iotdb 控制层
 * date: 2022/8/15 21:50
 * author: zhouhong
 */
@Log4j2
@RestController
public class IotDbController {

    @Resource
    private IotDbServer iotDbServer;
    @Resource
    private IotDBSessionConfig iotDBSessionConfig;

    /**
     * 插入数据
     * @param vehicle
     */
    @PostMapping("/api/device/insert")
    public ResponseData insert(@RequestBody Vehicle vehicle) throws StatementExecutionException, ServerException, IoTDBConnectionException {
        iotDbServer.insertData(vehicle);
        return ResponseData.success();
    }

    /**
     * 查询数据
     * @param vehicle
     */
    @PostMapping("/api/device/queryData")
    public ResponseData queryDataFromIotDb(@RequestBody VehicleReq vehicle) throws Exception {
        return ResponseData.success(iotDbServer.queryDataFromIotDb(vehicle));
    }

    /**
     * 删除分组
     * @return
     */
    @PostMapping("/api/device/deleteGroup")
    public ResponseData deleteGroup() throws StatementExecutionException, IoTDBConnectionException {
        iotDBSessionConfig.deleteStorageGroup("root.a1eaKSRpRty");
        iotDBSessionConfig.deleteStorageGroup("root.smartretirement");
        return ResponseData.success();
    }

}
