package com.mobai.iotDB.service.impl;

import com.mobai.domain.Vehicle;
import com.mobai.iotDB.config.IotDBSessionConfig;
import com.mobai.iotDB.service.IotDbServer;
import com.mobai.req.VehicleReq;
import lombok.extern.log4j.Log4j2;
import org.apache.iotdb.rpc.IoTDBConnectionException;
import org.apache.iotdb.rpc.StatementExecutionException;
import org.apache.iotdb.session.SessionDataSet;
import org.apache.iotdb.tsfile.read.common.Field;
import org.apache.iotdb.tsfile.read.common.RowRecord;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.math.BigDecimal;
import java.rmi.ServerException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * description: iot服务实现类
 * date: 2022/8/15 9:43
 * author: zhouhong
 */

@Log4j2
@Service
public class IotDbServerImpl implements IotDbServer {

    @Resource
    private IotDBSessionConfig iotDBSessionConfig;


    @Override
    public void insertData(Vehicle vehicle) throws StatementExecutionException, ServerException, IoTDBConnectionException {
        // iotDbParam: 模拟设备上报消息
        // bizkey: 业务唯一key  PK :产品唯一编码   SN:设备唯一编码
        String deviceId = "root.vin." + vehicle.getVin();
        // 将设备上报的数据存入数据库(时序数据库)
        List<String> measurementsList = new ArrayList<>() {{
            add("startTime");
            add("longitude");
            add("latitude");
            add("speed");
            add("mileage");
            add("voltage");
            add("current");
            add("resistance");
            add("gear");
            add("accelerationPedal");
            add("brakePedal");
            add("fuelConsumptionRate");
            add("motorControllerTemperature");
            add("motorSpeed");
            add("motoTorque");
            add("motorTemperature");
            add("motorVoltage");
            add("motorCurrent");
            add("remainingBattery");
            add("maximumFeedbackPower");
            add("maximumDischargePower");
            add("selfCheckCounter");
            add("totalBatteryCurrent");
            add("totalBatteryVoltage");
            add("singleBatteryMaxVoltage");
            add("singleBatteryMinVoltage");
            add("singleBatteryMaxTemperature");
            add("singleBatteryMinTemperature");
            add("availableBatteryCapacity");
            add("vehicleStatus");
            add("chargingStatus");
            add("operatingStatus");
            add("chargingEnergyStorageStatus");
            add("driveMotorStatus");
            add("positionStatus");
            add("easStatus");
            add("ptcStatus");
            add("epsStatus");
            add("absStatus");
            add("mcuStatus");
            add("heatingStatus");
            add("batteryStatus");
            add("batteryInsulationStatus");
            add("dcdcStatus");
            add("chgStatus");
        }};
        //车辆具体数据
        List<String> valuesList = new ArrayList<>() {{
            add(String.valueOf(vehicle.getStartTime()));
            add(String.valueOf(vehicle.getLongitude()));
            add(String.valueOf(vehicle.getLatitude()));
            add(String.valueOf(vehicle.getSpeed()));
            add(String.valueOf(vehicle.getMileage()));
            add(String.valueOf(vehicle.getVoltage()));
            add(String.valueOf(vehicle.getCurrent()));
            add(String.valueOf(vehicle.getResistance()));
            add(String.valueOf(vehicle.getGear()));
            add(String.valueOf(vehicle.getAccelerationPedal()));
            add(String.valueOf(vehicle.getBrakePedal()));
            add(String.valueOf(vehicle.getFuelConsumptionRate()));
            add(String.valueOf(vehicle.getMotorControllerTemperature()));
            add(String.valueOf(vehicle.getMotorSpeed()));
            add(String.valueOf(vehicle.getMotoTorque()));
            add(String.valueOf(vehicle.getMotorTemperature()));
            add(String.valueOf(vehicle.getMotorVoltage()));
            add(String.valueOf(vehicle.getMotorCurrent()));
            add(String.valueOf(vehicle.getRemainingBattery()));
            add(String.valueOf(vehicle.getMaximumFeedbackPower()));
            add(String.valueOf(vehicle.getMaximumDischargePower()));
            add(String.valueOf(vehicle.getSelfCheckCounter()));
            add(String.valueOf(vehicle.getTotalBatteryCurrent()));
            add(String.valueOf(vehicle.getTotalBatteryVoltage()));
            add(String.valueOf(vehicle.getSingleBatteryMaxVoltage()));
            add(String.valueOf(vehicle.getSingleBatteryMinVoltage()));
            add(String.valueOf(vehicle.getSingleBatteryMaxTemperature()));
            add(String.valueOf(vehicle.getSingleBatteryMinTemperature()));
            add(String.valueOf(vehicle.getAvailableBatteryCapacity()));
            add(String.valueOf(vehicle.getVehicleStatus()));
            add(String.valueOf(vehicle.getChargingStatus()));
            add(String.valueOf(vehicle.getOperatingStatus()));
            add(String.valueOf(vehicle.getChargingEnergyStorageStatus()));
            add(String.valueOf(vehicle.getDriveMotorStatus()));
            add(String.valueOf(vehicle.getPositionStatus()));
            add(String.valueOf(vehicle.getEasStatus()));
            add(String.valueOf(vehicle.getPtcStatus()));
            add(String.valueOf(vehicle.getEpsStatus()));
            add(String.valueOf(vehicle.getAbsStatus()));
            add(String.valueOf(vehicle.getMcuStatus()));
            add(String.valueOf(vehicle.getHeatingStatus()));
            add(String.valueOf(vehicle.getBatteryStatus()));
            add(String.valueOf(vehicle.getBatteryInsulationStatus()));
            add(String.valueOf(vehicle.getDcdcStatus()));
            add(String.valueOf(vehicle.getChgStatus()));
        }};

        iotDBSessionConfig.insertRecord(deviceId, vehicle.getStartTime(), measurementsList, valuesList);
    }


    @Override
    public List<Vehicle> queryDataFromIotDb(VehicleReq req) throws Exception {
        List<Vehicle> iotDbResultList = new ArrayList<>();

        if (null != req.getVin()) {
            String sql = "select * from " + "root.vin." + req.getVin();
            // 开始时间
            if (req.getStartTime() != null) {
                sql += " where startTime >= " + req.getStartTime();
            }
            // 结束时间
            if (req.getStartTime() != null) {
                sql += " and startTime < " + req.getEndTime();
            }
            SessionDataSet sessionDataSet = iotDBSessionConfig.query(sql);
            List<String> columnNames = sessionDataSet.getColumnNames();
            List<String> titleList = new ArrayList<>();
            // 排除Time字段 -- 方便后面后面拼装数据
            for (int i = 1; i < columnNames.size(); i++) {
                String[] temp = columnNames.get(i).split("\\.");
                titleList.add(temp[temp.length - 1]);
            }
            // 封装处理数据
            packagingData(req, iotDbResultList, sessionDataSet, titleList);
        } else {
            log.info("VIN不能为空！！");
        }
        return iotDbResultList;
    }

    /**
     * 封装处理数据
     *
     * @param req
     * @param iotDbResultList
     * @param sessionDataSet
     * @param titleList
     * @throws StatementExecutionException
     * @throws IoTDBConnectionException
     */
    private void packagingData(VehicleReq req, List<Vehicle> iotDbResultList, SessionDataSet sessionDataSet, List<String> titleList)
            throws StatementExecutionException, IoTDBConnectionException {
        int fetchSize = sessionDataSet.getFetchSize();
        if (fetchSize > 0) {
            while (sessionDataSet.hasNext()) {
                Vehicle vehicle = new Vehicle();
                RowRecord next = sessionDataSet.next();
                List<Field> fields = next.getFields();
                vehicle.setStartTime(next.getTimestamp());
                Map<String, String> map = new HashMap<>();

                for (int i = 0; i < fields.size(); i++) {

                    Field field = fields.get(i);
                    // 这里的需要按照类型获取
                    map.put(titleList.get(i), field.getObjectValue(field.getDataType()).toString());
                }
                log.info(map);
//                vehicle.setStartTime(Long.valueOf(map.get("startTime")));
                vehicle.setVin(map.get("vin"));
                vehicle.setSpeed(new BigDecimal(map.get("speed")));
                vehicle.setLatitude(new BigDecimal(map.get("latitude")));
                vehicle.setLongitude(new BigDecimal(map.get("longitude")));
                vehicle.setMileage(new BigDecimal(map.get("mileage")));
                vehicle.setGear(map.get("gear"));
                vehicle.setVehicleStatus(new BigDecimal(map.get("vehicleStatus")).intValue());
                vehicle.setChargingStatus(new BigDecimal(map.get("chargingStatus")).intValue());
                vehicle.setChargingEnergyStorageStatus(new BigDecimal(map.get("chargingEnergyStorageStatus")).intValue());
                vehicle.setEasStatus(new BigDecimal(map.get("easStatus")).intValue());
                vehicle.setMotorControllerTemperature(new BigDecimal(map.get("motorControllerTemperature")));
                vehicle.setTotalBatteryCurrent(new BigDecimal(map.get("totalBatteryCurrent")));
                vehicle.setSingleBatteryMaxVoltage(new BigDecimal(map.get("singleBatteryMaxVoltage")));
                vehicle.setOperatingStatus(new BigDecimal(map.get("operatingStatus")).intValue());
                vehicle.setHeatingStatus(new BigDecimal(map.get("heatingStatus")).intValue());
                vehicle.setDcdcStatus(new BigDecimal(map.get("dcdcStatus")).intValue());
                vehicle.setDriveMotorStatus(new BigDecimal(map.get("driveMotorStatus")).intValue());
                vehicle.setPositionStatus(new BigDecimal(map.get("positionStatus")).intValue());
                vehicle.setPtcStatus(new BigDecimal(map.get("ptcStatus")).intValue());
                vehicle.setEpsStatus(new BigDecimal(map.get("epsStatus")).intValue());
                vehicle.setAbsStatus(new BigDecimal(map.get("absStatus")).intValue());
                vehicle.setMcuStatus(new BigDecimal(map.get("mcuStatus")).intValue());
                vehicle.setBatteryInsulationStatus(new BigDecimal(map.get("batteryInsulationStatus")).intValue());
                vehicle.setBatteryStatus(new BigDecimal(map.get("batteryStatus")).intValue());
                vehicle.setChgStatus(new BigDecimal(map.get("chgStatus")).intValue());
                vehicle.setTotalBatteryVoltage(new BigDecimal(map.get("totalBatteryVoltage")));
                vehicle.setMotorSpeed(new BigDecimal(map.get("motorSpeed")));
                vehicle.setMotorCurrent(new BigDecimal(map.get("motorCurrent")));
                vehicle.setMotorVoltage(new BigDecimal(map.get("motorVoltage")));
                vehicle.setAccelerationPedal(new BigDecimal(map.get("accelerationPedal")));
                vehicle.setBrakePedal(new BigDecimal(map.get("brakePedal")));
                vehicle.setSelfCheckCounter(new BigDecimal(map.get("selfCheckCounter")));
                vehicle.setMotorTemperature(new BigDecimal(map.get("motorTemperature")));
                vehicle.setMaximumDischargePower(new BigDecimal(map.get("maximumDischargePower")));
                vehicle.setMaximumFeedbackPower(new BigDecimal(map.get("maximumFeedbackPower")));
                vehicle.setFuelConsumptionRate(new BigDecimal(map.get("fuelConsumptionRate")));
                vehicle.setVin(req.getVin());
                vehicle.setVoltage(new BigDecimal(map.get("voltage")));
                vehicle.setCurrent(new BigDecimal(map.get("current")));
                vehicle.setResistance(new BigDecimal(map.get("resistance")));
                vehicle.setMotoTorque(new BigDecimal(map.get("motoTorque")));
                vehicle.setRemainingBattery(new BigDecimal(map.get("remainingBattery")));
                vehicle.setSingleBatteryMinVoltage(new BigDecimal(map.get("singleBatteryMinVoltage")));
                vehicle.setSingleBatteryMaxTemperature(new BigDecimal(map.get("singleBatteryMaxTemperature")));
                vehicle.setSingleBatteryMinTemperature(new BigDecimal(map.get("singleBatteryMinTemperature")));
                vehicle.setAvailableBatteryCapacity(new BigDecimal(map.get("availableBatteryCapacity")));
                iotDbResultList.add(vehicle);
            }
        }
    }
}
