package com.mobai.kafka.listener;

import com.mobai.domain.MqttServerModel;
import com.mobai.domain.Vehicle;
import com.mobai.forest.ForestGet;
import org.apache.kafka.clients.consumer.ConsumerRebalanceListener;
import org.apache.kafka.clients.consumer.ConsumerRecord;
import org.apache.kafka.clients.consumer.ConsumerRecords;
import org.apache.kafka.clients.consumer.KafkaConsumer;
import org.apache.kafka.common.TopicPartition;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.autoconfigure.SpringBootApplication;
import org.springframework.stereotype.Component;

import java.time.Duration;
import java.util.ArrayList;
import java.util.Collection;
import java.util.List;
import java.util.Properties;

/**
 * @author Saisai
 * @className VinConsumer
 * @description 描述
 * @date 2024/6/18 17:18
 */

@Component
public class VinConsumer {

    public static void main(String[] args) {

        ArrayList<TopicPartition> topicPartitions = new ArrayList<>();
        List<String> topics = forestGet.getIps().getData().stream().map(MqttServerModel::getTopic).toList();
        for (String topic : topics) {
            for (int i = 0; i < 8; i++) {
                TopicPartition topicPartition = new TopicPartition(topic, i);
                topicPartitions.add(topicPartition);
            }
        }

        new ConsumerRebalanceListener(){
            @Override
            public void onPartitionsRevoked(Collection<TopicPartition> partitions) {
                // 可以在这里处理分区被撤销前的逻辑
                System.out.println("Partitions revoked: " + partitions);
            }

            @Override
            public void onPartitionsAssigned(Collection<TopicPartition> partitions) {
                // 可以在这里处理分区被分配后的逻辑
                System.out.println("Partitions assigned: " + partitions);
            }
        };
//        1.参数配置:不是每一非得配置
        Properties props = new Properties();
        props.put("bootstrap.servers", "localhost:9092");
        props.put("auto.commit.interval.ms", "1000");
        //因为每一个消费者必须属于某一个消费者组，所以必须还设置group.id
        props.put("group.id", "test");
        props.put("enable.auto.commit", "true");
        props.put("session.timeout.ms", "30000");
        props.put("key.deserializer", "org.apache.kafka.common.serialization.StringDeserializer");
        props.put("value.deserializer", "org.apache.kafka.common.serialization.StringDeserializer");
        KafkaConsumer<String, String> consumer = new KafkaConsumer<>(props);
        consumer.assign(topicPartitions);
        try {
//            while (true) {
                ConsumerRecords<String, String> records = consumer.poll(Duration.ofMillis(100)); // 假设等待100毫秒获取消息
                if (!records.isEmpty()) { // 检查是否有消息到来
                    // 创建线程异步执行
                    new Thread(() -> {
                        for (TopicPartition partition : records.partitions()) {
                            List<ConsumerRecord<String, String>> partitionRecords = records.records(partition);
                            for (ConsumerRecord<String, String> record : partitionRecords) {
                                // 处理每条消息
                                log.info("Offset = {}, Key = {}, Value = {}", record.offset(), record.key(), record.value());
                                Vehicle vehicle = getVehicle(record.value());
                                if (redisService.hasKey(vehicle.getVin())){
                                    redisService.setList(vehicle.getVin(),JSON.toJSONString(vehicle));
                                }
                            }
                        }
                    }).start();
                } else {
                    // 当没有消息时，选择休眠一小段时间避免过度占用CPU，或者执行其他逻辑
                    Thread.sleep(10);
                }
//            }
        } catch (InterruptedException e) {
            // 处理解除阻塞时的中断异常，如Thread.sleep被中断
            Thread.currentThread().interrupt(); // 重新设置中断标志
            log.error("Consumer was interrupted.", e);
        }
        finally {
            consumer.close();
        }
    }
}
