package com.mobai.kafka.listener;

import com.alibaba.fastjson2.JSON;
import com.alibaba.fastjson2.JSONObject;
import com.mobai.domain.MqttServerModel;
import com.mobai.domain.Vehicle;
import com.mobai.forest.ForestGet;
import com.mobai.iotDB.service.IotDbServer;
import com.mobai.utils.RedisService;
import lombok.extern.slf4j.Slf4j;
import org.apache.iotdb.rpc.IoTDBConnectionException;
import org.apache.iotdb.rpc.StatementExecutionException;
import org.apache.kafka.clients.consumer.ConsumerRebalanceListener;
import org.apache.kafka.clients.consumer.ConsumerRecord;
import org.apache.kafka.clients.consumer.ConsumerRecords;
import org.apache.kafka.clients.consumer.KafkaConsumer;
import org.apache.kafka.common.TopicPartition;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.annotation.Bean;
import org.springframework.kafka.annotation.KafkaListener;

import org.springframework.stereotype.Component;

import java.lang.annotation.Annotation;
import java.math.BigDecimal;
import java.rmi.ServerException;
import java.time.Duration;
import java.util.ArrayList;
import java.util.Collection;
import java.util.List;
import java.util.Properties;

/**
 * @description: kafka 消费者
 * @copyright: @Copyright (c) 2022
 * @company: hmblogs
 * @author: heming
 * @version: 1.0.0
 * @createTime: 2024-01-18 8:31
 */
@Component
@Slf4j
public class kafkaConsumerListenerExample {


    @Autowired
    private RedisService redisService;


    @Autowired
    private IotDbServer iotDbServer;

//    @Autowired
//    private KafkaConsumer<String, String> consumer;

    @Autowired
    private ForestGet forestGet;

    /**
     * 监听消费
     *
     * @param record
     */
    @KafkaListener(topics = {"topic0", "topic1"}, groupId = "topics")
    public void consume(ConsumerRecord<String, String> record) {
        System.out.println(record);
        Vehicle vehicle = getVehicle(record.value());
        // 存入iotDB数据库
        try {
            iotDbServer.insertData(vehicle);
            log.info("添加成功");
            if (redisService.hasKey(vehicle.getVin())) {
                redisService.setList(vehicle.getVin(), JSON.toJSONString(vehicle));
            }
        } catch (StatementExecutionException e) {
            throw new RuntimeException(e);
        } catch (ServerException e) {
            throw new RuntimeException(e);
        } catch (IoTDBConnectionException e) {
            throw new RuntimeException(e);
        }
    }

////        批量消费
//    @KafkaListener( topics = {"topic0","topic1"}, groupId = "Topics")
//    public void onBatchMessage(List<ConsumerRecord<String, String>> records) {
//        System.out.println(">>> 批量消费一次，recoreds.size()=" + records.size());
//        List<Vehicle> collect = records.stream().jsonObject(record -> {
//            Vehicle value = JSON.parseObject(record.value(), Vehicle.class);
//            log.info("车辆报文：{}", value);
//            return value;
//        }).toList();
//        log.warn("批量消费的数量为：{}，结果为:{}", records.size(), collect);
//        System.out.println(records);
//    }

    @Bean
    public void partitionConsumer() {
        ArrayList<TopicPartition> topicPartitions = new ArrayList<>();
        List<String> topics = forestGet.getIps().getData().stream().map(MqttServerModel::getTopic).toList();
        for (String topic : topics) {
            for (int i = 0; i < 8; i++) {
                TopicPartition topicPartition = new TopicPartition(topic, i);
                topicPartitions.add(topicPartition);
            }
        }
//
//        new ConsumerRebalanceListener (){
//            @Override
//            public void onPartitionsRevoked(Collection<TopicPartition> partitions) {
//                // 可以在这里处理分区被撤销前的逻辑
//                System.out.println("Partitions revoked: " + partitions);
//            }
//
//            @Override
//            public void onPartitionsAssigned(Collection<TopicPartition> partitions) {
//                // 可以在这里处理分区被分配后的逻辑
//                System.out.println("Partitions assigned: " + partitions);
//            }
//        };
//        //1.参数配置:不是每一非得配置
//        Properties props = new Properties();
//        props.put("bootstrap.servers", "localhost:9092");
//        props.put("auto.commit.interval.ms", "1000");
//        //因为每一个消费者必须属于某一个消费者组，所以必须还设置group.id
//        props.put("group.id", "test");
//        props.put("enable.auto.commit", "true");
//        props.put("session.timeout.ms", "30000");
//        props.put("key.deserializer", "org.apache.kafka.common.serialization.StringDeserializer");
//        props.put("value.deserializer", "org.apache.kafka.common.serialization.StringDeserializer");
//        KafkaConsumer<String, String> consumer = new KafkaConsumer<>(props);
//        consumer.assign(topicPartitions);
//        try {
////            while (true) {
//                ConsumerRecords<String, String> records = consumer.poll(Duration.ofMillis(100)); // 假设等待100毫秒获取消息
//                if (!records.isEmpty()) { // 检查是否有消息到来
//                    // 创建线程异步执行
//                    new Thread(() -> {
//                        for (TopicPartition partition : records.partitions()) {
//                            List<ConsumerRecord<String, String>> partitionRecords = records.records(partition);
//                            for (ConsumerRecord<String, String> record : partitionRecords) {
//                                // 处理每条消息
//                                log.info("Offset = {}, Key = {}, Value = {}", record.offset(), record.key(), record.value());
//                                Vehicle vehicle = getVehicle(record.value());
//                                if (redisService.hasKey(vehicle.getVin())){
//                                    redisService.setList(vehicle.getVin(),JSON.toJSONString(vehicle));
//                                }
//                            }
//                        }
//                    }).start();
//                } else {
//                    // 当没有消息时，选择休眠一小段时间避免过度占用CPU，或者执行其他逻辑
//                    Thread.sleep(10);
//                }
////            }
//        } catch (InterruptedException e) {
//            // 处理解除阻塞时的中断异常，如Thread.sleep被中断
//            Thread.currentThread().interrupt(); // 重新设置中断标志
//            log.error("Consumer was interrupted.", e);
//        }
//        finally {
//            consumer.close();
//        }

    }

    // 自定义分区分配监听器
    public static class MyPartitionAssignmentListener implements ConsumerRebalanceListener {
        @Override
        public void onPartitionsRevoked(Collection<TopicPartition> partitions) {
            // 可以在这里处理分区被撤销前的逻辑
            System.out.println("Partitions revoked: " + partitions);
        }

        @Override
        public void onPartitionsAssigned(Collection<TopicPartition> partitions) {
            // 可以在这里处理分区被分配后的逻辑
            System.out.println("Partitions assigned: " + partitions);
        }
    }

    /**
     * 解析kafka拉取数据
     *
     * @param jsonString
     * @return 车辆信息
     */
    public Vehicle getVehicle(String jsonString) {
        JSONObject jsonObject = JSONObject.parseObject(jsonString);
        // 解析车辆报文对象
        Vehicle vehicle = new Vehicle() {{
            setStartTime(Long.valueOf(jsonObject.get("startTime").toString()));
            setSpeed(new BigDecimal(jsonObject.get("speed").toString()));
            setLatitude(new BigDecimal(jsonObject.get("latitude").toString()));
            setLongitude(new BigDecimal(jsonObject.get("longitude").toString()));
            setMileage(new BigDecimal(jsonObject.get("mileage").toString()));
            setGear(jsonObject.get("gear").toString());
            setVehicleStatus(new BigDecimal(jsonObject.get("vehicleStatus").toString()).intValue());
            setChargingStatus(new BigDecimal(jsonObject.get("chargingStatus").toString()).intValue());
            setChargingEnergyStorageStatus(new BigDecimal(jsonObject.get("chargingEnergyStorageStatus").toString()).intValue());
            setEasStatus(new BigDecimal(jsonObject.get("easStatus").toString()).intValue());
            setMotorControllerTemperature(new BigDecimal(jsonObject.get("motorControllerTemperature").toString()));
            setTotalBatteryCurrent(new BigDecimal(jsonObject.get("totalBatteryCurrent").toString()));
            setSingleBatteryMaxVoltage(new BigDecimal(jsonObject.get("singleBatteryMaxVoltage").toString()));
            setOperatingStatus(new BigDecimal(jsonObject.get("operatingStatus").toString()).intValue());
            setHeatingStatus(new BigDecimal(jsonObject.get("heatingStatus").toString()).intValue());
            setDcdcStatus(new BigDecimal(jsonObject.get("dcdcStatus").toString()).intValue());
            setDriveMotorStatus(new BigDecimal(jsonObject.get("driveMotorStatus").toString()).intValue());
            setPositionStatus(new BigDecimal(jsonObject.get("positionStatus").toString()).intValue());
            setPtcStatus(new BigDecimal(jsonObject.get("ptcStatus").toString()).intValue());
            setEpsStatus(new BigDecimal(jsonObject.get("epsStatus").toString()).intValue());
            setAbsStatus(new BigDecimal(jsonObject.get("absStatus").toString()).intValue());
            setMcuStatus(new BigDecimal(jsonObject.get("mcuStatus").toString()).intValue());
            setBatteryInsulationStatus(new BigDecimal(jsonObject.get("batteryInsulationStatus").toString()).intValue());
            setBatteryStatus(new BigDecimal(jsonObject.get("batteryStatus").toString()).intValue());
            setChgStatus(new BigDecimal(jsonObject.get("chgStatus").toString()).intValue());
            setTotalBatteryVoltage(new BigDecimal(jsonObject.get("totalBatteryVoltage").toString()));
            setMotorSpeed(new BigDecimal(jsonObject.get("motorSpeed").toString()));
            setMotorCurrent(new BigDecimal(jsonObject.get("motorCurrent").toString()));
            setMotorVoltage(new BigDecimal(jsonObject.get("motorVoltage").toString()));
            setAccelerationPedal(new BigDecimal(jsonObject.get("accelerationPedal").toString()));
            setBrakePedal(new BigDecimal(jsonObject.get("brakePedal").toString()));
            setSelfCheckCounter(new BigDecimal(jsonObject.get("selfCheckCounter").toString()));
            setMotorTemperature(new BigDecimal(jsonObject.get("motorTemperature").toString()));
            setMaximumDischargePower(new BigDecimal(jsonObject.get("maximumDischargePower").toString()));
            setMaximumFeedbackPower(new BigDecimal(jsonObject.get("maximumFeedbackPower").toString()));
            setFuelConsumptionRate(new BigDecimal(jsonObject.get("fuelConsumptionRate").toString()));
            setVin(jsonObject.get("vin").toString());
            setVoltage(new BigDecimal(jsonObject.get("voltage").toString()));
            setCurrent(new BigDecimal(jsonObject.get("current").toString()));
            setResistance(new BigDecimal(jsonObject.get("resistance").toString()));
            setMotoTorque(new BigDecimal(jsonObject.get("motoTorque").toString()));
            setRemainingBattery(new BigDecimal(jsonObject.get("remainingBattery").toString()));
            setSingleBatteryMinVoltage(new BigDecimal(jsonObject.get("singleBatteryMinVoltage").toString()));
            setSingleBatteryMaxTemperature(new BigDecimal(jsonObject.get("singleBatteryMaxTemperature").toString()));
            setSingleBatteryMinTemperature(new BigDecimal(jsonObject.get("singleBatteryMinTemperature").toString()));
            setAvailableBatteryCapacity(new BigDecimal(jsonObject.get("availableBatteryCapacity").toString()));

        }};

//        if (redisService.hasKey(vehicle.getVin())) {
//            redisService.setList(vehicle.getVin(), JSON.toJSONString(vehicle));
//        }
        // 进行消息处理逻辑
        log.info("车辆报文信息 ： " + vehicle);
        return vehicle;
    }
}

